install.packages("magic")
install.packages("geometry")
install.packages("spatstat")
install.packages("sp") 
install.packages("rgdal")
 
library(magic)
library(geometry)
library(spatstat)
library(sp) 
library(rgdal)
 
lambda = 4

M=2*lambda

N1 <- 5  # Choose N1=10 for Figures 3 and 4 
N2 <- 10 # Choose N2=50 for Figures 3 and 4 

P1 <- matrix(0,M+1) 
V1 <- matrix(0,M+1) 

P2 <- matrix(0,M+1) 
V2 <- matrix(0,M+1) 

A1 <- matrix(0,N1,N2)
B1 <- matrix(0,N1) 

for (i1 in 1:N1){
for (i2 in 1:N2){
Z = rpoispp(lambda) 
X <- cbind(Z$x,Z$y);
if (length(X) <= 4) A1[i1,i2]=0 # length(X)/2
else
A1[i1,i2]=length(convhulln(X, "FA")$p)/2 -length(chull(X)) # length(convhulln(X, "FA")$hull)/2
}
}

for (n in 0:M){

for (i1 in 1:N1) B1[i1]=length(which(A1[i1,]==n))/N2

P1[n+1] = mean(B1)
V1[n+1] = var(B1)

print(c(n,P1[n+1],V1[n+1]))

}

for (n in 0:M){

A2 <- matrix(0,N1,N2)
B2 <- matrix(0,N1) 

for (i1 in 1:N1){
for (i2 in 1:N2){
Z = rpoispp(lambda) 
X <- cbind(Z$x,Z$y);
if (length(X) <=4) A2[i1,i2]=0^n
else A2[i1,i2]=(lambda*convhulln(X, "FA")$vol)^n*exp(-lambda*convhulln(X, "FA")$vol)
}
}

for (i1 in 1:N1) B2[i1]=sum(A2[i1,])/N2/factorial(n)

P2[n+1] = mean(B2)
V2[n+1] = var(B2)

print(c(n,P1[n+1],V1[n+1]))
print(c(n,P2[n+1],V2[n+1]))

}

plot(P1,col="darkorange",xlab="n",cex=1.5,ylab="Probability p",axes=FALSE)
points(P2,col="blue",cex=1.5,pch=16) 
lines(P2,col="blue",lw=4)
lines(P1,col="darkorange",lty=2, lw=4) 
points(dpois(x=0:M,lambda),col="red",lty=2, lw=4)
lines(dpois(x=0:M,lambda),col="red",lty=2, lw=4)
legend("topright",legend=c("Sampling","Averaging",expression(paste("Poisson(",lambda,")"))),col=c("darkorange","blue","red"),lw=3,lty=c(2,1,2),cex=1.8)
# legend("topright",legend=c("Sampling","Averaging"),pch=c(1,1),col=c("darkorange","blue"))
axis(1,at=seq(1,M+1,1),labels=seq(0,M,1)) 
axis(2) 

